require 'spec_helper'

describe Statham::Attribute do
  describe '#serialize' do
    context 'when type is boolean' do
      let(:attribute) { Statham::Attribute.new(type: :boolean) }

      it { expect(attribute.serialize(nil)).to be_nil }
      it { expect(attribute.serialize(true)).to eq(true) }
      it { expect(attribute.serialize(false)).to eq(false) }
      it { expect(attribute.serialize('1')).to eq(true) }
      it { expect(attribute.serialize('0')).to eq(false) }
    end

    context 'when type is array' do
      let(:attribute) { Statham::Attribute.new(type: :array) }

      it { expect(attribute.serialize(nil)).to be_nil }
      it { expect { attribute.serialize(true) }.to raise_error }
      it { expect { attribute.serialize('string') }.to raise_error }
      it { expect { attribute.serialize([1, 2, 3]) }.not_to raise_error }
    end

    context 'when type is hash' do
      let(:attribute) { Statham::Attribute.new(type: :hash) }

      it { expect(attribute.serialize(nil)).to be_nil }
      it { expect { attribute.serialize(true) }.to raise_error }
      it { expect { attribute.serialize({ foo: :bar }) }.not_to raise_error }
      it { expect { attribute.serialize('string') }.to raise_error }
      it { expect { attribute.serialize([1, 2, 3]) }.to raise_error }
    end

    context 'when type is float' do
      let(:attribute) { Statham::Attribute.new(type: :float) }

      it { expect(attribute.serialize(nil)).to be_nil }
      it { expect { attribute.serialize(true) }.to raise_error }
      it { expect { attribute.serialize({ foo: :bar }) }.to raise_error }
      it { expect { attribute.serialize('string') }.not_to raise_error }
      it { expect(attribute.serialize('string')).to eq(0) }
      it { expect { attribute.serialize([1, 2, 3]) }.to raise_error }
      it { expect { attribute.serialize(1) }.not_to raise_error }
      it { expect(attribute.serialize(1)).to eq(1.to_f) }
    end

    context 'when default value is set' do
      let(:default) { 'hello' }
      let(:value) { 'world' }
      let(:attribute) { Statham::Attribute.new(default: default) }

      it { expect(attribute.serialize(nil)).to eq(default) }
      it { expect(attribute.serialize(value)).to eq(value) }
    end
  end
end
