require 'spec_helper'

class AttributesTestModel < ActiveRecord::Base
  include Statham::Document

  statham :foo do |json|
    json.attribute :bar
  end
end

class ChildTestModel < AttributesTestModel
  statham :foo do |json|
    json.attribute :baz
  end
end

class ChildChildTestModel < ChildTestModel
  statham :foo do |json|
    json.attribute :qux
  end
end

describe Statham::Document do
  describe '.statham_attribute_sets' do
    it { expect(AttributesTestModel.statham_attribute_sets).to be_a(Hash) }
    it { expect(AttributesTestModel.statham_attribute_sets.keys).to eq([:foo]) }
    it { expect(ChildTestModel.statham_attribute_sets).to be_a(Hash) }
    it { expect(ChildTestModel.statham_attribute_sets.keys).to eq([:foo]) }
  end

  describe '.parent_statham_attribute_set_for' do
    it { expect(ChildTestModel.parent_statham_attribute_set_for(:foo)).to eq(AttributesTestModel.statham_attribute_sets[:foo]) }
    it { expect(ChildChildTestModel.parent_statham_attribute_set_for(:foo)).to eq(ChildTestModel.statham_attribute_sets[:foo]) }
  end

  describe '.apply_statham_attribute_set' do
    let(:parent_class) { Class.new { def self.serialize(*args); end } }
    let(:attribute_set) { Statham::AttributeSet.new }
    let(:attributes) { { foo: { }, bar: { } } }

    before { parent_class.include Statham::Document }
    before { attributes.each { |name, options| attribute_set.attribute(name, options) } }

    it { expect(parent_class).to receive(:serialize).once; parent_class.apply_statham_attribute_set(attribute_set) }
    it { expect { parent_class.apply_statham_attribute_set(attribute_set) }.to change { parent_class.new.methods } }
    it { expect { parent_class.apply_statham_attribute_set(attribute_set) }.to change { parent_class.new.methods.sort }.by([:foo, :foo=, :bar, :bar=].sort) }
  end
end
