require 'spec_helper'

describe Statham::Serializer do
  let(:attribute_set) { Statham::AttributeSet.new(name: :foobar) }

  describe '#dump' do
    let(:statham_column) { Statham::Serializer.new(attribute_set) }
    let(:object) { { foo: 'bar' } }

    it { expect(statham_column.dump(object)).to eq('{"foo":"bar"}') }
    it { expect(ActiveSupport::JSON.decode(statham_column.dump(object))).to eq({ 'foo' => 'bar' }) }

    context 'when attribute_set has attributes' do
      before do
        attribute_set.attribute :baz
      end

      it { expect(ActiveSupport::JSON.decode(statham_column.dump(object))).to eq({ 'foo' => 'bar' }) }
    end

    context 'when attribute_set has attributes with default value' do
      let(:baz_default) { 'bazbaz' }

      before do
        attribute_set.attribute :baz, default: baz_default
      end

      it { expect(ActiveSupport::JSON.decode(statham_column.dump(object))).to eq({ 'foo' => 'bar', 'baz' => baz_default }) }
    end
  end

  describe '#load' do
    let(:statham_column) { Statham::Serializer.new(attribute_set) }
    let(:object_json) { ActiveSupport::JSON.encode({ foo: 'bar' }) }
    let(:baz_default) { 'bazbaz' }

    before do
      attribute_set.attribute :baz, default: baz_default
    end

    context 'when object_json does not have attribute with default value' do
      it { expect(statham_column.load(object_json)).to eq({ 'foo' => 'bar', 'baz' => baz_default }) }
      it { expect(statham_column.load(object_json)[:foo]).to eq('bar') }
      it { expect(statham_column.load(object_json)[:baz]).to eq(baz_default) }
    end

    context 'when object_json has attribute with default value' do
      let(:object_json) { ActiveSupport::JSON.encode({ foo: 'bar', baz: nil }) }

      it { expect(statham_column.load(object_json)).to eq({ 'foo' => 'bar', 'baz' => nil }) }
      it { expect(statham_column.load(object_json)[:foo]).to eq('bar') }
      it { expect(statham_column.load(object_json)[:baz]).to be_nil }
    end

    context 'when no default value is set' do
      let(:baz_default) { nil }

      it { expect(statham_column.load(object_json)).to eq({ 'foo' => 'bar' }) }
      it { expect(statham_column.load(object_json)[:foo]).to eq('bar') }
      it { expect(statham_column.load(object_json)[:baz]).to be_nil }
    end
  end
end
